@@ -8,9 +8,9 @@ from matplotlib.colors import hsv_to_rgb
 from scipy import stats
 
 from . import io
-from ..detection.masks import create_cell_pix, create_neuropil_masks, create_cell_mask
+from ..extraction import masks
 from ..detection.stats import roi_stats
-from ..extraction.extract import extract_traces_from_masks
+from ..extraction import extract_traces_from_masks
 from ..extraction.dcnv import oasis
 
 
@@ -33,12 +33,12 @@ def masks_and_traces(ops, stat_manual, stat_orig):
         stat_all.append(stat_orig[n])
     stat_all = roi_stats(stat_all, dy, dx, ops['Ly'], ops['Lx'])
     cell_masks = [
-        create_cell_mask(stat, Ly=ops['Ly'], Lx=ops['Lx'], allow_overlap=ops['allow_overlap']) for stat in stat_all
+        masks.create_cell_mask(stat, Ly=ops['Ly'], Lx=ops['Lx'], allow_overlap=ops['allow_overlap']) for stat in stat_all
     ]
-    cell_pix = create_cell_pix(stat_all, Ly=ops['Ly'], Lx=ops['Lx'], allow_overlap=ops['allow_overlap'])
+    cell_pix = masks.create_cell_pix(stat_all, Ly=ops['Ly'], Lx=ops['Lx'])
     manual_roi_stats = stat_all[:len(stat_manual)]
     manual_cell_masks = cell_masks[:len(stat_manual)]
-    manual_neuropil_masks = create_neuropil_masks(
+    manual_neuropil_masks = masks.create_neuropil_masks(
         ypixs=[stat['ypix'] for stat in manual_roi_stats],
         xpixs=[stat['xpix'] for stat in manual_roi_stats],
         cell_pix=cell_pix,
@@ -392,11 +392,12 @@ class ROIDraw(QtGui.QMainWindow):
             ellipse = self.ROIs[n].ellipse
             yrange = self.ROIs[n].yrange
             xrange = self.ROIs[n].xrange
+            med = self.ROIs[n].med
             x, y = np.meshgrid(xrange, yrange)
             ypix = y[ellipse].flatten()
             xpix = x[ellipse].flatten()
             lam = np.ones(ypix.shape)
-            stat0.append({'ypix': ypix, 'xpix': xpix, 'lam': lam, 'npix': ypix.size})
+            stat0.append({'ypix': ypix, 'xpix': xpix, 'lam': lam, 'npix': ypix.size, 'med': med})
             self.tlabel.append(pg.TextItem(str(n), self.ROIs[n].color, anchor=(0, 0)))
             self.tlabel[-1].setPos(xpix.mean(), ypix.mean())
             self.p0.addItem(self.tlabel[-1])
@@ -494,6 +495,7 @@ class sROI():
         self.ROI.addScaleHandle([1, 0.5], [0., 0.5])
         self.ROI.addScaleHandle([0.5, 0], [0.5, 1])
         self.ROI.setAcceptedMouseButtons(QtCore.Qt.LeftButton)
+        self.med = [imy, imx]
         parent.p0.addItem(self.ROI)
 
     def remove(self, parent):
