@@ -20,6 +20,7 @@ def make_masks_and_enable_buttons(parent):
 
     parent.loadBeh.setEnabled(True)
     parent.saveMat.setEnabled(True)
+    parent.saveNWB.setEnabled(True)
     parent.saveMerge.setEnabled(True)
     parent.sugMerge.setEnabled(True)
     parent.manual.setEnabled(True)
@@ -85,7 +86,7 @@ def make_masks_and_enable_buttons(parent):
     graphics.init_range(parent)
     traces.plot_trace(parent)
     parent.xyrat = 1.0
-    if (isinstance(parent.ops['diameter'], list) and
+    if (isinstance(parent.ops['diameter'], (list, np.ndarray)) and
         len(parent.ops['diameter'])>1 and
         parent.ops['aspect']==1.0):
         parent.xyrat = parent.ops["diameter"][0] / parent.ops["diameter"][1]
@@ -320,7 +321,8 @@ def load_to_GUI(parent, basename, procs):
     parent.hasred = hasred
     parent.notmerged = np.ones_like(parent.iscell).astype(np.bool)
     for n in range(len(parent.stat)):
-        parent.stat[n]['chan2_prob'] = parent.probredcell[n]
+        if parent.hasred:
+            parent.stat[n]['chan2_prob'] = parent.probredcell[n]
         parent.stat[n]['inmerge'] = 0
     parent.stat = np.array(parent.stat)
     make_masks_and_enable_buttons(parent)
@@ -366,7 +368,7 @@ def load_behavior(parent):
             parent.beh_resampled = resample_frames(parent.beh, parent.beh_time, np.arange(0,parent.Fcell.shape[1]))
         else:
             parent.beh_resampled = parent.beh
-        b = len(parent.colors)
+        b = 8
         parent.colorbtns.button(b).setEnabled(True)
         parent.colorbtns.button(b).setStyleSheet(parent.styleUnpressed)
         masks.beh_masks(parent)
@@ -421,15 +423,17 @@ def save_merge(parent):
     np.save(os.path.join(parent.basename, 'stat.npy'), parent.stat)
     np.save(os.path.join(parent.basename, 'F.npy'), parent.Fcell)
     np.save(os.path.join(parent.basename, 'Fneu.npy'), parent.Fneu)
-    np.save(os.path.join(parent.basename, 'F_chan2.npy'), parent.F_chan2)
-    np.save(os.path.join(parent.basename, 'Fneu_chan2.npy'), parent.Fneu_chan2)
+    if parent.hasred:
+        np.save(os.path.join(parent.basename, 'F_chan2.npy'), parent.F_chan2)
+        np.save(os.path.join(parent.basename, 'Fneu_chan2.npy'), parent.Fneu_chan2)
+        np.save(os.path.join(parent.basename, 'redcell.npy'),
+                np.concatenate((np.expand_dims(parent.redcell,axis=1),
+                np.expand_dims(parent.probredcell,axis=1)), axis=1))
     np.save(os.path.join(parent.basename, 'spks.npy'), parent.Spks)
     iscell =  np.concatenate((parent.iscell[:,np.newaxis],
                               parent.probcell[:,np.newaxis]), axis=1)
     np.save(os.path.join(parent.basename, 'iscell.npy'), iscell)
-    np.save(os.path.join(parent.basename, 'redcell.npy'),
-            np.concatenate((np.expand_dims(parent.redcell,axis=1),
-            np.expand_dims(parent.probredcell,axis=1)), axis=1))
+    
     parent.notmerged = np.ones(parent.iscell.size, np.bool)
 
 def load_custom_mask(parent):
