@@ -1,7 +1,4 @@
-import glob
-import json
-import os
-import shutil
+import glob, json, os, shutil, pathlib
 from datetime import datetime
 
 import numpy as np
@@ -47,6 +44,7 @@ class RunWindow(QtGui.QDialog):
         self.win.setLayout(self.layout)
         # initial ops values
         self.opsfile = parent.opsuser
+        self.ops_path = os.fspath(pathlib.Path.home().joinpath('.suite2p').joinpath('ops'))
         try:
             self.reset_ops()
             print('loaded default ops')
@@ -87,14 +85,14 @@ class RunWindow(QtGui.QDialog):
                      'save_mat', 'save_NWB' 'combined', '1Preg', 'nonrigid', 
                     'connected', 'roidetect', 'neuropil_extract', 
                     'spikedetect', 'keep_movie_raw', 'allow_overlap', 'sparse_mode']
-        tifkeys = ['nplanes','nchannels','functional_chan','tau','fs','do_bidiphase','bidiphase', 'multiplane_parallel']
+        tifkeys = ['nplanes','nchannels','functional_chan','tau','fs','do_bidiphase','bidiphase', 'multiplane_parallel', 'ignore_flyback']
         outkeys = ['preclassify','save_mat','save_NWB','combined','reg_tif','reg_tif_chan2','aspect','delete_bin','move_bin']
         regkeys = ['do_registration','align_by_chan','nimg_init','batch_size','smooth_sigma', 'smooth_sigma_time','maxregshift','th_badframes','keep_movie_raw','two_step_registration']
         nrkeys = [['nonrigid','block_size','snr_thresh','maxregshiftNR'], ['1Preg','spatial_hp_reg','pre_smooth','spatial_taper']]
-        cellkeys = ['roidetect','sparse_mode','anatomical_only', 'diameter','spatial_scale','connected','threshold_scaling','max_overlap','max_iterations','high_pass']
-        neudeconvkeys = [['neuropil_extract', 'allow_overlap','inner_neuropil_radius','min_neuropil_pixels'], ['spikedetect','win_baseline','sig_baseline','neucoeff']]
+        cellkeys = ['roidetect', 'denoise', 'anatomical_only', 'diameter', 'spatial_scale', 'threshold_scaling', 'max_overlap','max_iterations','high_pass']
+        neudeconvkeys = [['neuropil_extract', 'allow_overlap','inner_neuropil_radius','min_neuropil_pixels'], ['soma_crop','spikedetect','win_baseline','sig_baseline','neucoeff']]
         keys = [tifkeys, outkeys, regkeys, nrkeys, cellkeys, neudeconvkeys]
-        labels = ['Main settings','Output settings','Registration',['Nonrigid','1P'],'ROI detection',['Extraction/Neuropil','Deconvolution']]
+        labels = ['Main settings','Output settings','Registration',['Nonrigid','1P'],'ROI detection',['Extraction/Neuropil','Classification/Deconvolution']]
         tooltips = ['each tiff has this many planes in sequence',
                     'each tiff has this many channels per plane',
                     'this channel is used to extract functional ROIs (1-based)',
@@ -103,6 +101,7 @@ class RunWindow(QtGui.QDialog):
                     'whether or not to compute bidirectional phase offset of recording (from line scanning)',
                     'set a fixed number (in pixels) for the bidirectional phase offset',
                     'process each plane with a separate job on a computing cluster',
+                    'ignore flyback planes 0-indexed separated by a comma e.g. "0,10"; "-1" means no planes ignored so all planes processed',
                     'apply ROI classifier before signal extraction with probability threshold (set to 0 to turn off)',
                     'save output also as mat file "Fall.mat"',
                     'save output also as NWB file "ophys.nwb"',
@@ -131,11 +130,10 @@ class RunWindow(QtGui.QDialog):
                     'whether to smooth before high-pass filtering before registration',
                     "how much to ignore on edges (important for vignetted windows, for FFT padding do not set BELOW 3*smooth_sigma)",
                     'if 1, run cell (ROI) detection',
-                    'whether to run sparse_mode cell extraction (scale-free) or original algorithm (default is original)',
-                    'run cellpose to get masks on 1: max_proj / mean_img; or 2: mean_img',
-                    'if sparse_mode=0, input average diameter of ROIs in recording (can give a list e.g. 6,9)',
-                    'if sparse_mode=1, choose size of ROIs: 0 = multi-scale; 1 = 6 pixels, 2 = 12, 3 = 24, 4 = 48',
-                    'whether or not to require ROIs to be fully connected (set to 0 for dendrites/boutons)',
+                    'if 1, run PCA denoising on binned movie to improve cell detection',
+                    'run cellpose to get masks on 1: max_proj / mean_img; 2: mean_img; or 3: mean_img enhanced',
+                    'if anatomical_only>0, input average diameter of ROIs in recording (can give a list e.g. 6,9)',
+                    'if anatomical_only=0, choose size of ROIs: 0 = multi-scale; 1 = 6 pixels, 2 = 12, 3 = 24, 4 = 48',
                     'adjust the automatically determined threshold for finding ROIs by this scalar multiplier',
                     'ROIs with greater than this overlap as a fraction of total pixels will be discarded',
                     'maximum number of iterations for ROI detection',
@@ -144,6 +142,7 @@ class RunWindow(QtGui.QDialog):
                     'allow shared pixels to be used for fluorescence extraction from overlapping ROIs (otherwise excluded from both ROIs)',
                     'number of pixels between ROI and neuropil donut',
                     'minimum number of pixels in the neuropil',
+                    'if 1, crop dendrites for cell classification stats like compactness',
                     'if 1, run spike detection (deconvolution)',
                     'window for maximin',
                     'smoothing constant for gaussian filter',
@@ -198,7 +197,7 @@ class RunWindow(QtGui.QDialog):
                 k+=1
                 for key in keyl[kl]:
                     lops = 1
-                    if self.ops[key] or (self.ops[key] == 0):
+                    if self.ops[key] or (self.ops[key] == 0) or len(self.ops[key])==0:
                         qedit = LineEdit(wk,key,self)
                         qlabel = QtGui.QLabel(key)
                         qlabel.setToolTip(tooltips[kk])
@@ -344,8 +343,8 @@ class RunWindow(QtGui.QDialog):
         self.f = 0
         self.compile_ops_db()
         L = len(self.opslist)
-        np.save('ops%d.npy'%L, self.ops)
-        np.save('db%d.npy'%L, self.db)
+        np.save(os.path.join(self.ops_path, 'ops%d.npy'%L), self.ops)
+        np.save(os.path.join(self.ops_path, 'db%d.npy'%L), self.db)
         self.opslist.append('ops%d.npy'%L)
         if hasattr(self, 'h5_key') and len(self.h5_key) > 0:
             self.db['h5py_key'] = self.h5_key
@@ -378,13 +377,15 @@ class RunWindow(QtGui.QDialog):
             self.add_ops()
         self.finish = True
         self.error = False
-        shutil.copy('ops%d.npy'%self.f, 'ops.npy')
-        shutil.copy('db%d.npy'%self.f, 'db.npy')
-        self.db = np.load('db.npy', allow_pickle=True).item()
+        ops_file = os.path.join(self.ops_path, 'ops.npy')
+        db_file = os.path.join(self.ops_path, 'db.npy')
+        shutil.copy(os.path.join(self.ops_path, 'ops%d.npy'%self.f), ops_file)
+        shutil.copy(os.path.join(self.ops_path, 'db%d.npy'%self.f), db_file)
+        self.db = np.load(db_file, allow_pickle=True).item()
         print('Running suite2p!')
         print('starting process')
         print(self.db)
-        self.process.start('python -u -W ignore -m suite2p --ops ops.npy --db db.npy')
+        self.process.start('python -u -W ignore -m suite2p --ops %s --db %s'%(ops_file, db_file))
 
     def stop(self):
         self.finish = False
@@ -460,10 +461,11 @@ class RunWindow(QtGui.QDialog):
 
     def load_ops(self, name=None):
         print('loading ops')
-        if name is None:
+        if not (isinstance(name, str) and len(name)>0):
             name = QtGui.QFileDialog.getOpenFileName(self, 'Open ops file (npy or json)')
             name = name[0]
-        if len(name)>0:
+        
+        if len(name) > 0:
             ext = os.path.splitext(name)[1]
             try:
                 if ext == '.npy':
@@ -601,6 +603,12 @@ class LineEdit(QtGui.QLineEdit):
                 okey = [int(diams[0]), int(diams[1])]
             else:
                 okey = int(diams[0])
+        elif key=='ignore_flyback':
+            okey = self.text().replace(' ','').split(',')
+            for i in range(len(okey)):
+                okey[i] = int(okey[i])
+            if len(okey)==1 and okey[0]==-1:
+                okey = []
         else:
             if key in intkeys:
                 okey = int(float(self.text()))
@@ -617,6 +625,17 @@ class LineEdit(QtGui.QLineEdit):
                 dstr = str(int(ops[key][0])) + ', ' + str(int(ops[key][1]))
             else:
                 dstr = str(int(ops[key]))
+        elif key=='ignore_flyback':
+            if not isinstance(ops[key], (list, np.ndarray)):
+                ops[key] = [ops[key]]
+            if len(ops[key])==0:
+                dstr = '-1'
+            else:
+                dstr = ''
+                for i in ops[key]:
+                    dstr+= str(int(ops[key][i]))
+                    if i<len(ops[key])-1:
+                        dstr+=', '
         else:
             if type(ops[key]) is not bool:
                 dstr = str(ops[key])
