@@ -20,9 +20,13 @@ def get_list_of_data(outputs_to_check, output_dir) -> Iterator[np.ndarray]:
 def compare_list_of_outputs(output_name_list, data_list_one, data_list_two) -> Iterator[bool]:
     for output, data1, data2 in zip(output_name_list, data_list_one, data_list_two):
         if output == 'stat':  # where the elements of npy arrays are dictionaries (e.g: stat.npy)
+            if data2[0]['lam'].sum() > 1.0:
+                for d in data2:
+                    d['lam'] /= d['lam'].sum()
             for gt_dict, output_dict in zip(data1, data2):
                 for k in gt_dict.keys():
-                    yield np.allclose(gt_dict[k], output_dict[k], rtol=1e-4, atol=5e-2)
+                    if k=='ypix' or k=='xpix' or k=='lam':
+                        yield np.allclose(gt_dict[k], output_dict[k], rtol=1e-4, atol=5e-2)
         elif output == 'iscell':  # just check the first column; are cells/noncells classified the same way?
             yield np.array_equal(data1[:, 0], data2[:, 0])
         elif output == 'redcell':
